@@ -18,6 +18,7 @@ macro_rules! impl_exception_boilerplate {
         }
 
         impl $name {
+            /// Creates a new [PyErr](crate::PyErr) of this type.
             pub fn new_err<A>(args: A) -> $crate::PyErr
             where
                 A: $crate::PyErrArguments + Send + Sync + 'static,
@@ -275,6 +276,7 @@ impl_native_exception!(PyIOError, PyExc_IOError);
 impl_native_exception!(PyWindowsError, PyExc_WindowsError);
 
 impl PyUnicodeDecodeError {
+    /// Creates a Python `UnicodeDecodeError`.
     pub fn new<'p>(
         py: Python<'p>,
         encoding: &CStr,
@@ -294,6 +296,7 @@ impl PyUnicodeDecodeError {
         }
     }
 
+    /// Creates a Python `UnicodeDecodeError` from a Rust UTF-8 decoding error.
     pub fn new_utf8<'p>(
         py: Python<'p>,
         input: &[u8],
@@ -329,7 +332,7 @@ pub mod socket {
 }
 
 #[cfg(test)]
-mod test {
+mod tests {
     use super::{PyException, PyUnicodeDecodeError};
     use crate::types::{IntoPyDict, PyDict};
     use crate::{PyErr, Python};
@@ -399,7 +402,7 @@ mod test {
         let error_type = py.get_type::<CustomError>();
         let ctx = [("CustomError", error_type)].into_py_dict(py);
         let type_description: String = py
-            .eval("str(CustomError)", None, Some(&ctx))
+            .eval("str(CustomError)", None, Some(ctx))
             .unwrap()
             .extract()
             .unwrap();
@@ -407,7 +410,7 @@ mod test {
         py.run(
             "assert CustomError('oops').args == ('oops',)",
             None,
-            Some(&ctx),
+            Some(ctx),
         )
         .unwrap();
     }
