@@ -1,6 +1,6 @@
 // Copyright (c) 2017-present PyO3 Project and Contributors
 
-//! Interaction with python's global interpreter lock
+//! Interaction with Python's global interpreter lock
 
 use crate::{ffi, internal_tricks::Unsendable, Python};
 use parking_lot::{const_mutex, Mutex, Once};
@@ -14,21 +14,21 @@ use std::{
 static START: Once = Once::new();
 
 thread_local! {
-    /// This is a internal counter in pyo3 monitoring whether this thread has the GIL.
+    /// This is an internal counter in pyo3 monitoring whether this thread has the GIL.
     ///
     /// It will be incremented whenever a GILGuard or GILPool is created, and decremented whenever
     /// they are dropped.
     ///
     /// As a result, if this thread has the GIL, GIL_COUNT is greater than zero.
     ///
-    /// pub(crate) because it is manipulated temporarily by Python::allow_threads
+    /// pub(crate) because it is manipulated temporarily by `Python::allow_threads`.
     pub(crate) static GIL_COUNT: Cell<usize> = Cell::new(0);
 
-    /// Temporally hold objects that will be released when the GILPool drops.
+    /// Temporarily hold objects that will be released when the GILPool drops.
     static OWNED_OBJECTS: RefCell<Vec<NonNull<ffi::PyObject>>> = RefCell::new(Vec::with_capacity(256));
 }
 
-/// Check whether the GIL is acquired.
+/// Checks whether the GIL is acquired.
 ///
 /// Note: This uses pyo3's internal count rather than PyGILState_Check for two reasons:
 ///  1) for performance
@@ -40,21 +40,22 @@ pub(crate) fn gil_is_acquired() -> bool {
 
 /// Prepares the use of Python in a free-threaded context.
 ///
-/// If the Python interpreter is not already initialized, this function
-/// will initialize it with disabled signal handling
-/// (Python will not raise the `KeyboardInterrupt` exception).
-/// Python signal handling depends on the notion of a 'main thread', which must be
-/// the thread that initializes the Python interpreter.
+/// If the Python interpreter is not already initialized, this function will initialize it with
+/// signal handling disabled (Python will not raise the `KeyboardInterrupt` exception). Python
+/// signal handling depends on the notion of a 'main thread', which must be the thread that
+/// initializes the Python interpreter.
 ///
-/// If both the Python interpreter and Python threading are already initialized,
-/// this function has no effect.
+/// If both the Python interpreter and Python threading are already initialized, this function has
+/// no effect.
+///
+/// This function is unavailable under PyPy because PyPy cannot be embedded in Rust (or any other
+/// software). Support for this is tracked on the
+/// [PyPy issue tracker](https://foss.heptapod.net/pypy/pypy/-/issues/3286).
 ///
 /// # Panics
-/// - If the Python interpreter is initialized but Python threading is not,
-///   a panic occurs.
-///   It is not possible to safely access the Python runtime unless the main
-///   thread (the thread which originally initialized Python) also initializes
-///   threading.
+/// - If the Python interpreter is initialized but Python threading is not, a panic occurs.
+///   It is not possible to safely access the Python runtime unless the main thread (the thread
+///   which originally initialized Python) also initializes threading.
 ///
 /// # Examples
 /// ```rust
@@ -174,7 +175,7 @@ where
 
 /// RAII type that represents the Global Interpreter Lock acquisition.
 ///
-/// Users are strongly encouraged to [`Python::with_gil`](struct.Python.html#method.with_gil)
+/// Users are strongly encouraged to use [`Python::with_gil`](struct.Python.html#method.with_gil)
 /// instead of directly constructing this type.
 /// See [`Python::acquire_gil`](struct.Python.html#method.acquire_gil) for more.
 ///
@@ -240,6 +241,15 @@ impl GILGuard {
             }
         }
 
+        Self::acquire_unchecked()
+    }
+
+    /// Acquires the `GILGuard` without performing any state checking.
+    ///
+    /// This can be called in "unsafe" contexts where the normal interpreter state
+    /// checking performed by `GILGuard::acquire` may fail. This includes calling
+    /// as part of multi-phase interpreter initialization.
+    pub(crate) fn acquire_unchecked() -> GILGuard {
         let gstate = unsafe { ffi::PyGILState_Ensure() }; // acquire GIL
 
         // If there's already a GILPool, we should not create another or this could lead to
@@ -355,7 +365,7 @@ pub struct GILPool {
 }
 
 impl GILPool {
-    /// Create a new `GILPool`. This function should only ever be called with the GIL.
+    /// Creates a new `GILPool`. This function should only ever be called with the GIL held.
     ///
     /// It is recommended not to use this API directly, but instead to use `Python::new_pool`, as
     /// that guarantees the GIL is held.
@@ -402,8 +412,8 @@ impl Drop for GILPool {
     }
 }
 
-/// Register a Python object pointer inside the release pool, to have reference count increased
-/// next time the GIL is acquired in pyo3.
+/// Registers a Python object pointer inside the release pool, to have its reference count increased
+/// the next time the GIL is acquired in pyo3.
 ///
 /// If the GIL is held, the reference count will be increased immediately instead of being queued
 /// for later.
@@ -418,8 +428,8 @@ pub unsafe fn register_incref(obj: NonNull<ffi::PyObject>) {
     }
 }
 
-/// Register a Python object pointer inside the release pool, to have reference count decreased
-/// next time the GIL is acquired in pyo3.
+/// Registers a Python object pointer inside the release pool, to have its reference count decreased
+/// the next time the GIL is acquired in pyo3.
 ///
 /// If the GIL is held, the reference count will be decreased immediately instead of being queued
 /// for later.
@@ -434,7 +444,7 @@ pub unsafe fn register_decref(obj: NonNull<ffi::PyObject>) {
     }
 }
 
-/// Register an owned object inside the GILPool.
+/// Registers an owned object inside the GILPool, to be released when the GILPool drops.
 ///
 /// # Safety
 /// The object must be an owned Python reference.
@@ -444,14 +454,14 @@ pub unsafe fn register_owned(_py: Python, obj: NonNull<ffi::PyObject>) {
     let _ = OWNED_OBJECTS.try_with(|holder| holder.borrow_mut().push(obj));
 }
 
-/// Increment pyo3's internal GIL count - to be called whenever GILPool or GILGuard is created.
+/// Increments pyo3's internal GIL count - to be called whenever GILPool or GILGuard is created.
 #[inline(always)]
 fn increment_gil_count() {
     // Ignores the error in case this function called from `atexit`.
     let _ = GIL_COUNT.try_with(|c| c.set(c.get() + 1));
 }
 
-/// Decrement pyo3's internal GIL count - to be called whenever GILPool or GILGuard is dropped.
+/// Decrements pyo3's internal GIL count - to be called whenever GILPool or GILGuard is dropped.
 #[inline(always)]
 fn decrement_gil_count() {
     // Ignores the error in case this function called from `atexit`.
@@ -465,7 +475,7 @@ fn decrement_gil_count() {
     });
 }
 
-/// Ensure the GIL is held, used in the implementation of Python::with_gil
+/// Ensures the GIL is held, used in the implementation of `Python::with_gil`.
 pub(crate) fn ensure_gil() -> EnsureGIL {
     if gil_is_acquired() {
         EnsureGIL(None)
@@ -474,6 +484,18 @@ pub(crate) fn ensure_gil() -> EnsureGIL {
     }
 }
 
+/// Ensures the GIL is held, without interpreter state checking.
+///
+/// This bypasses interpreter state checking that would normally be performed
+/// before acquiring the GIL.
+pub(crate) fn ensure_gil_unchecked() -> EnsureGIL {
+    if gil_is_acquired() {
+        EnsureGIL(None)
+    } else {
+        EnsureGIL(Some(GILGuard::acquire_unchecked()))
+    }
+}
+
 /// Struct used internally which avoids acquiring the GIL where it's not necessary.
 #[allow(clippy::upper_case_acronyms)]
 pub(crate) struct EnsureGIL(Option<GILGuard>);
@@ -495,7 +517,7 @@ impl EnsureGIL {
 }
 
 #[cfg(test)]
-mod test {
+mod tests {
     use super::{gil_is_acquired, GILPool, GIL_COUNT, OWNED_OBJECTS, POOL};
     use crate::{ffi, gil, AsPyPointer, IntoPyPointer, PyObject, Python, ToPyObject};
     use std::ptr::NonNull;
