@@ -16,9 +16,13 @@ use std::os::raw::{c_char, c_int};
 /// See [Python::version].
 #[derive(Debug)]
 pub struct PythonVersionInfo<'p> {
+    /// Python major version (e.g. `3`).
     pub major: u8,
+    /// Python minor version (e.g. `11`).
     pub minor: u8,
+    /// Python patch version (e.g. `0`).
     pub patch: u8,
+    /// Python version suffix, if applicable (e.g. `a0`).
     pub suffix: Option<&'p str>,
 }
 
@@ -156,6 +160,31 @@ impl Python<'_> {
     {
         f(unsafe { gil::ensure_gil().python() })
     }
+
+    /// Like [Python::with_gil] except Python interpreter state checking is skipped.
+    ///
+    /// Normally when the GIL is acquired, we check that the Python interpreter is an
+    /// appropriate state (e.g. it is fully initialized). This function skips those
+    /// checks.
+    ///
+    /// # Safety
+    ///
+    /// If [Python::with_gil] would succeed, it is safe to call this function.
+    ///
+    /// In most cases, you should use [Python::with_gil].
+    ///
+    /// A justified scenario for calling this function is during multi-phase interpreter
+    /// initialization when [Python::with_gil] would fail before `_Py_InitializeMain()`
+    /// is called because the interpreter is only partially initialized.
+    ///
+    /// Behavior in other scenarios is not documented.
+    #[inline]
+    pub unsafe fn with_gil_unchecked<F, R>(f: F) -> R
+    where
+        F: for<'p> FnOnce(Python<'p>) -> R,
+    {
+        f(gil::ensure_gil_unchecked().python())
+    }
 }
 
 impl<'p> Python<'p> {
@@ -367,6 +396,7 @@ impl<'p> Python<'p> {
                 return Err(PyErr::fetch(self));
             }
             let res_ptr = ffi::PyEval_EvalCode(code_obj, globals, locals);
+            ffi::Py_DECREF(code_obj);
 
             self.from_owned_ptr_or_err(res_ptr)
         }
@@ -674,7 +704,7 @@ impl<'p> Python<'p> {
 }
 
 #[cfg(test)]
-mod test {
+mod tests {
     use super::*;
     use crate::types::{IntoPyDict, PyList};
 
