@@ -127,6 +127,7 @@ impl PyModule {
             }
 
             let mptr = ffi::PyImport_ExecCodeModuleEx(module.as_ptr(), cptr, filename.as_ptr());
+            ffi::Py_DECREF(cptr);
             if mptr.is_null() {
                 return Err(PyErr::fetch(py));
             }
@@ -272,7 +273,7 @@ impl PyModule {
     /// make an *instance* of `Foo` (or *get* one for that matter, as we haven't exported
     /// anything that can return instances of `Foo`).
     ///
-    /// [1]: https://pyo3.rs/main/class.html#constructor
+    /// [1]: https://pyo3.rs/latest/class.html#constructor
     pub fn add_class<T>(&self) -> PyResult<()>
     where
         T: PyClass,
@@ -420,7 +421,7 @@ impl PyModule {
 }
 
 #[cfg(test)]
-mod test {
+mod tests {
     use crate::{types::PyModule, Python};
 
     #[test]
